"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  default: () => src_default
});
module.exports = __toCommonJS(src_exports);
var import_path = __toESM(require("path"));

// src/utils/checker.ts
var defaultAcceptedJsxIdentifiers = ["View", "Text", "Image"];
var checker_default = () => ({
  isAcceptedJsx(node, acceptedJsxIdentifiers = defaultAcceptedJsxIdentifiers) {
    return node.name.type === "JSXIdentifier" && acceptedJsxIdentifiers.indexOf(node.name.name) > -1;
  },
  hasClassNameProp(node) {
    const classNameAttr = node.attributes.find(
      (a) => a.type === "JSXAttribute" && a.name.name === "className"
    );
    if (!classNameAttr)
      return false;
    return true;
  },
  hasImportedTw: (programPath) => {
    let isTwImported = false;
    programPath.traverse({
      ImportDefaultSpecifier({ node }) {
        if (node.local.name === "tw")
          isTwImported = true;
      }
    });
    return isTwImported;
  }
});

// src/utils/creator.ts
var creator_default = (t) => ({
  createTwStylesFromString: (className) => {
    return t.taggedTemplateExpression(
      t.identifier("tw"),
      t.templateLiteral(
        [
          t.templateElement({
            raw: className,
            cooked: className
          })
        ],
        []
      )
    );
  },
  createTwStylesFromTml: (tpl) => {
    return t.taggedTemplateExpression(t.identifier("tw"), tpl);
  },
  createTwStylesObj: (twStyleList) => {
    return t.variableDeclaration("const", [
      t.variableDeclarator(
        t.identifier("twStyles"),
        t.objectExpression(
          twStyleList.map(
            (c) => t.objectProperty(t.identifier(c.classId), c.style)
          )
        )
      )
    ]);
  },
  createImportTw: (relativePath) => {
    return t.importDeclaration(
      [t.importDefaultSpecifier(t.identifier("tw"))],
      t.stringLiteral(relativePath)
    );
  },
  createStyleExpression: (classId) => {
    return t.memberExpression(t.identifier("twStyles"), t.identifier(classId));
  },
  createStyleExpressionContainer: (classId) => {
    return t.jSXExpressionContainer(
      t.memberExpression(t.identifier("twStyles"), t.identifier(classId))
    );
  }
});

// src/utils/getter.ts
var getter_default = (t) => ({
  getCombinedExpressionContainer(...containers) {
    const updatedStyleExpContainer = containers.reduce((prevContainer, currContainer) => {
      if (!prevContainer)
        return currContainer;
      if (prevContainer.expression.type === "JSXEmptyExpression" || currContainer.expression.type === "JSXEmptyExpression")
        return prevContainer;
      const prevExp = prevContainer.expression;
      const currExp = currContainer.expression;
      if (prevExp.type === "ArrayExpression") {
        return t.jSXExpressionContainer(
          t.arrayExpression([currExp, ...prevExp.elements])
        );
      }
      if (currExp.type === "ArrayExpression") {
        return t.jSXExpressionContainer(
          t.arrayExpression([...currExp.elements, prevExp])
        );
      }
      return t.jSXExpressionContainer(t.arrayExpression([currExp, prevExp]));
    }, void 0);
    return updatedStyleExpContainer;
  }
});

// src/utils/handler.ts
var handler_default = (t, twStyleList) => ({
  handleClassName_StringLiteral(attrPath) {
    var _a;
    if (((_a = attrPath.node.value) == null ? void 0 : _a.type) !== "StringLiteral")
      return;
    const { createTwStylesFromString, createStyleExpressionContainer } = creator_default(t);
    const attrPathId = attrPath.scope.generateUidIdentifier("attr").name;
    const classId = `${attrPathId}_str`;
    const style = createTwStylesFromString(attrPath.node.value.value);
    if (style)
      twStyleList.push({ classId, style });
    attrPath.node.value = createStyleExpressionContainer(classId);
    attrPath.node.name.name = "style";
    return attrPath.node.value;
  },
  handleClassName_JSXExpressionContainer(attrPath) {
    var _a;
    if (((_a = attrPath.node.value) == null ? void 0 : _a.type) !== "JSXExpressionContainer")
      return;
    const { createTwStylesFromString, createStyleExpression, createStyleExpressionContainer, createTwStylesFromTml } = creator_default(t);
    const attrPathId = attrPath.scope.generateUidIdentifier("attr").name;
    const expression = attrPath.node.value.expression;
    switch (expression.type) {
      case "StringLiteral": {
        const classId = `${attrPathId}_exp_str`;
        const style = createTwStylesFromString(expression.value);
        if (style)
          twStyleList.push({ classId, style });
        attrPath.node.name.name = "style";
        attrPath.node.value = createStyleExpressionContainer(classId);
        return attrPath.node.value;
      }
      case "TemplateLiteral": {
        const classId = `${attrPathId}_exp_tpl`;
        twStyleList.push({ classId, style: createTwStylesFromTml(expression) });
        attrPath.node.name.name = "style";
        attrPath.node.value = createStyleExpressionContainer(classId);
        return attrPath.node.value;
      }
      case "ConditionalExpression": {
        attrPath.traverse({
          ConditionalExpression(conExpPath) {
            const node = conExpPath.node;
            const conExpPathId = conExpPath.scope.generateUidIdentifier(attrPathId).name + "_conExp";
            if (node.consequent.type === "StringLiteral") {
              const classId = `${conExpPathId}_conStr`;
              const style = createTwStylesFromString(node.consequent.value);
              style && twStyleList.push({ classId, style });
              node.consequent = createStyleExpression(classId);
            }
            if (node.alternate.type === "StringLiteral") {
              const classId = `${conExpPathId}_altStr`;
              const style = createTwStylesFromString(node.alternate.value);
              style && twStyleList.push({ classId, style });
              node.alternate = createStyleExpression(classId);
            }
            if (node.consequent.type === "TemplateLiteral") {
              const classId = `${conExpPathId}_conTml`;
              const style = createTwStylesFromTml(node.consequent);
              twStyleList.push({ classId, style });
              node.consequent = createStyleExpression(classId);
            }
            if (node.alternate.type === "TemplateLiteral") {
              const classId = `${conExpPathId}_altTml`;
              const style = createTwStylesFromTml(node.alternate);
              twStyleList.push({ classId, style });
              node.alternate = createStyleExpression(classId);
            }
          }
        });
        attrPath.node.name.name = "style";
        return attrPath.node.value;
      }
      default:
        break;
    }
  }
});

// src/index.ts
var src_default = ({ types: t }) => {
  return {
    visitor: {
      Program(programPath, programState) {
        const twPath = "twPath" in this.opts ? this.opts.twPath : void 0;
        const acceptedJsxIdentifiers = "acceptedJsxIdentifiers" in this.opts ? this.opts.acceptedJsxIdentifiers : void 0;
        const twStyleList = [];
        const { isAcceptedJsx, hasClassNameProp, hasImportedTw } = checker_default();
        const { createTwStylesObj, createImportTw } = creator_default(t);
        const { getCombinedExpressionContainer } = getter_default(t);
        const {
          handleClassName_StringLiteral,
          handleClassName_JSXExpressionContainer
        } = handler_default(t, twStyleList);
        programPath.traverse({
          JSXOpeningElement(jsxPath) {
            if (!isAcceptedJsx(jsxPath.node, acceptedJsxIdentifiers) || !hasClassNameProp(jsxPath.node))
              return;
            let oriStyle;
            jsxPath.traverse({
              JSXAttribute(attrPath) {
                var _a;
                if (attrPath.node.name.name === "style" && ((_a = attrPath.node.value) == null ? void 0 : _a.type) === "JSXExpressionContainer") {
                  oriStyle = attrPath.node.value;
                  attrPath.remove();
                }
              }
            });
            jsxPath.traverse({
              JSXAttribute(attrPath) {
                if (!attrPath.node.value || !attrPath.node.value.type)
                  return;
                if (attrPath.node.name.name === "className") {
                  let twExpContainer;
                  if (attrPath.node.value.type === "StringLiteral") {
                    twExpContainer = handleClassName_StringLiteral(attrPath);
                  } else if (attrPath.node.value.type === "JSXExpressionContainer") {
                    twExpContainer = handleClassName_JSXExpressionContainer(attrPath);
                  }
                  if (oriStyle && twExpContainer) {
                    const updatedStyleExpContainer = getCombinedExpressionContainer(oriStyle, twExpContainer);
                    attrPath.node.value = updatedStyleExpContainer;
                  }
                }
              }
            });
          }
        });
        if (twStyleList.length > 0) {
          const fileBody = programPath.node.body;
          fileBody.push(createTwStylesObj(twStyleList));
          if (hasImportedTw(programPath))
            return;
          const curr = programState.file.opts.filename;
          const root = programState.file.opts.cwd;
          let relativePath = "";
          if (!root || !curr || !twPath) {
            relativePath = "./lib/tw";
          } else {
            const twAbsPath = import_path.default.join(root, twPath);
            relativePath = `./${import_path.default.relative(import_path.default.dirname(curr), twAbsPath)}`;
          }
          fileBody.unshift(createImportTw(relativePath));
        }
      }
    }
  };
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
